//+------------------------------------------------------------------+
//|                                                  NonLagMA_v4.mq4 |
//|                                Copyright  2006, TrendLaboratory |
//|            http://finance.groups.yahoo.com/group/TrendLaboratory |
//|                                   E-mail: igorad2003@yahoo.co.uk |
//+------------------------------------------------------------------+
#property copyright "Copyright  2006, TrendLaboratory"
#property link      "http://finance.groups.yahoo.com/group/TrendLaboratory"


#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1  LimeGreen
#property indicator_width1  2
#property indicator_color2  PaleVioletRed
#property indicator_width2  2

//---- input parameters
extern string  TimeFrame      = "Current time frame";
extern int     Price          =  0;
extern int     Length         = 16;
extern int     Displace       =  0;
extern int     Filter         =  0;
extern double  Deviation      =  0;   

double Cycle =  4;

//---- indicator buffers
double MABuffer[];
double UpBuffer[];
double DnBuffer[];
double price[];
double trend[];
string indicatorFileName;
bool   returnBars;
bool   calculateValue;
int    timeFrame;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
  int init()
  {
   int ft=0;
   string short_name;
//---- indicator line
   IndicatorBuffers(5);
   SetIndexBuffer(0,UpBuffer); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,DnBuffer); SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2,MABuffer);
   SetIndexBuffer(3,price);
   SetIndexBuffer(4,trend);
   IndicatorDigits(MarketInfo(Symbol(),MODE_DIGITS));
//---- name for DataWindow and indicator subwindow label
   SetIndexLabel(0,"NLMA");
   SetIndexLabel(1,"Up");
   SetIndexLabel(2,"Dn");
//----
   SetIndexShift(0,Displace);
   SetIndexShift(1,Displace);
   SetIndexShift(2,Displace);
   
   SetIndexDrawBegin(0,Length*Cycle+Length);
   SetIndexDrawBegin(1,Length*Cycle+Length);
   SetIndexDrawBegin(2,Length*Cycle+Length);
//----
         indicatorFileName = WindowExpertName();
         returnBars        = (TimeFrame=="returnBars");     if (returnBars)     return(0);
         calculateValue    = (TimeFrame=="calculateValue"); if (calculateValue) return(0);
         timeFrame         = stringToTimeFrame(TimeFrame);
   short_name=timeFrameToString(timeFrame)+" NonLagMA("+Length+")";
   IndicatorShortName(short_name);
   return(0);
  }

//+------------------------------------------------------------------+
//| NonLagMA_v4                                                     |
//+------------------------------------------------------------------+
int start()
{
   int    i,shift, counted_bars=IndicatorCounted(),limit;
   double alfa, beta, t, Sum, Weight, g;
   double pi = 3.1415926535;    
   
   double Coeff =  3*pi;
   int Phase = Length-1;
   double Len = Length*Cycle + Phase;  
   
   if ( counted_bars > 0 )  limit=Bars-counted_bars;
   if ( counted_bars < 0 )  return(0);
   if ( counted_bars ==0 )  limit=Bars-Len-1; 
   if ( returnBars) { UpBuffer[0] = limit+1; return(0); }
   
   if (calculateValue || timeFrame==Period())   
   {
      if ( counted_bars < 1 ) 
      for(i=1;i<Length*Cycle+Length;i++) 
      {
         MABuffer[Bars-i]=0;    
         UpBuffer[Bars-i]=0;  
         DnBuffer[Bars-i]=0;  
      }
      for(shift=limit;shift>=0;shift--) 
      {	
      Weight=0; Sum=0; t=0;
         for (i=0;i<=Len-1;i++)
	      { 
            g = 1.0/(Coeff*t+1);   
            if (t <= 0.5 ) g = 1;
            beta = MathCos(pi*t);
            alfa = g * beta;
            //if (shift>=1) price[i] = iMA(NULL,0,Per,Displace,Mode,Price,shift+i); 
            //else 
            price[i] = iMA(NULL,0,1,0,MODE_SMA,Price,shift+i); 
            Sum += alfa*price[i];
            Weight += alfa;
            if ( t < 1 ) t += 1.0/(Phase-1); 
            else if ( t < Len-1 )  t += (2*Cycle-1)/(Cycle*Length-1);
         }
         if (Weight > 0) MABuffer[shift] = (1.0+Deviation/100)*Sum/Weight;
         if (Filter>0)
      {
      if( MathAbs(MABuffer[shift]-MABuffer[shift+1]) < Filter*Point ) MABuffer[shift]=MABuffer[shift+1];
      }
         trend[shift]=trend[shift+1];
         if (MABuffer[shift]-MABuffer[shift+1] > Filter*Point) trend[shift]= 1; 
         if (MABuffer[shift+1]-MABuffer[shift] > Filter*Point) trend[shift]=-1; 
         if (trend[shift]>0)
         {  
            UpBuffer[shift] = MABuffer[shift]-MABuffer[shift+1];
            DnBuffer[shift] = EMPTY_VALUE;
         }
         if (trend[shift]<0) 
         {
            DnBuffer[shift] = MABuffer[shift]-MABuffer[shift+1];
            UpBuffer[shift] = EMPTY_VALUE;
         }
      }
      return(0);	
   }

   //
   //
   //
   //
   //
      
   for(i = limit; i >=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         UpBuffer[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",Price,Length,Displace,Filter,Deviation,0,y);
         DnBuffer[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",Price,Length,Displace,Filter,Deviation,1,y);
  }
  return(0);
         
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}